__version__ = '1.8'

from .registrar import Registrar
from .directory import Directory
from .dining import Dining
from .dining import DiningV2
from .transit import Transit
from .news import News
from .laundry import Laundry
from .map import Map
from .studyspaces import StudySpaces
from .calendar3year import Calendar
from .fitness import Fitness
from .wharton import Wharton
